package app.mar.activities;

import android.os.Bundle;
import android.os.Handler;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.app.Activity;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

import com.google.zxing.ResultPoint;
import com.google.zxing.client.android.BeepManager;
import com.journeyapps.barcodescanner.BarcodeCallback;
import com.journeyapps.barcodescanner.BarcodeResult;
import com.journeyapps.barcodescanner.CaptureManager;
import com.journeyapps.barcodescanner.DecoratedBarcodeView;

import java.util.List;

import app.mar.utils.game.Game;
import app.mar.utils.game.challenges.Challenge;
import app.mar.utils.game.challenges.QCM;
import app.mar.utils.game.challenges.QRCodeTreasure;

public class QRCodeActivity extends AppCompatActivity  implements
        DecoratedBarcodeView.TorchListener, BarcodeCallback {

    private CaptureManager capture;
    private DecoratedBarcodeView barcodeScannerView;
    private Button switchFlashlightButton;
    private BeepManager beepManager;
    private Game mGame;
    public static final int REQUEST_CODE = 2001;
    private QRCodeTreasure mQr;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_qrcode);

        barcodeScannerView = (DecoratedBarcodeView)findViewById(R.id.zxing_barcode_scanner);
        barcodeScannerView.setTorchListener(this);
        beepManager=new BeepManager(this);

        switchFlashlightButton = (Button)findViewById(R.id.switch_flashlight);

        // if the device does not have flashlight in its camera,
        // then remove the switch flashlight button...
        if (!hasFlash()) {
            switchFlashlightButton.setVisibility(View.GONE);
        }


        mGame = Game.game();
        Challenge c = mGame.getCurrentChallenge();
        if(c instanceof QRCodeTreasure) {
            mQr = (QRCodeTreasure) c;
        }else
        {
            mQr=null;
        }

        capture = new CaptureManager(this, barcodeScannerView);
        capture.initializeFromIntent(getIntent(), savedInstanceState);
        barcodeScannerView.decodeContinuous(this);
    }

    @Override
    protected void onResume() {
        super.onResume();
        capture.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
        capture.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        capture.onDestroy();
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        capture.onSaveInstanceState(outState);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        return barcodeScannerView.onKeyDown(keyCode, event) || super.onKeyDown(keyCode, event);
    }

    /**
     * Check if the device's camera has a Flashlight.
     * @return true if there is Flashlight, otherwise false.
     */
    private boolean hasFlash() {
        return getApplicationContext().getPackageManager()
                .hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH);
    }

    public void switchFlashlight(View view) {
        if ("on".equals(switchFlashlightButton.getText())) {
            barcodeScannerView.setTorchOn();
        } else {
            barcodeScannerView.setTorchOff();
        }
    }

    @Override
    public void onTorchOn() {
        switchFlashlightButton.setText("Off");
    }

    @Override
    public void onTorchOff() {
        switchFlashlightButton.setText("On");
    }

    public void barcodeResult(final BarcodeResult result) {
        beepManager.playBeepSoundAndVibrate();

        Toast.makeText(this,"Text: '"+result.getText()+"'", Toast.LENGTH_LONG).show();

        if(result.getText().compareTo(mQr.getQRValue())==0)
        {
            barcodeScannerView.pause();
            setResult(RESULT_OK);
            finish();
        }else
        {
            Toast.makeText(this, mQr.getBadQRMessage(), Toast.LENGTH_LONG).show();
            barcodeScannerView.decodeContinuous(this);
        }

    }

    @Override
    public void possibleResultPoints(List<ResultPoint> resultPoints) {

    }
}
