package app.mar.utils.game.challenges;

import android.app.Activity;
import android.content.Intent;
import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import app.mar.activities.ARActivity;
import app.mar.activities.QuestionActivity;
import app.mar.utils.game.Game;

public abstract class Challenge implements Serializable{

    public static final String CHALLENGE_NULL="null";


    protected String mType=CHALLENGE_NULL;
    protected String mName="null";
    protected boolean mDone=false;
    protected Class  mStartWith = null;

    public Challenge(JSONObject root, Class start)
    {
        mStartWith=start;
        try {
            mType = root.getString("type");
            mName = root.getString("name");
            mDone = false;
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    public void exec(Game game, Activity act)
    {
        Intent intent = new Intent(act, mStartWith);
        intent.putExtra("challenge", this);
        intent.putExtra("game", game);
        Log.e("______","Starting class : '"+ mStartWith.getName()+"'");
        act.startActivityForResult(intent, getResultCode());
    }

    public String getType() {
        return mType;
    }

    public String getName() {
        return mName;
    }

    public abstract int getResultCode();

    public boolean isDone() {
        return mDone;
    }

    public void setDone()
    {
        mDone=true;
    }

    public static Class getChallengeClass(String classe)
    {
        try {
            Log.e("_____", "Trying to open class 'app.mar.utils.game.challenges."+classe+"'");
            Class cls = Class.forName("app.mar.utils.game.challenges."+classe);
            return cls;
        }
        catch(ClassNotFoundException cnfe)
        {
            cnfe.printStackTrace();
            return null;
        }
    }

    public static Constructor<?> getChallengeConstructor(String classe)
    {
        try {
            Class[] type = { Game.class, JSONObject.class, Activity.class };
            return getChallengeClass(classe).getConstructor(type);
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static Challenge callChallengeConstructor(String classe, Game g,JSONObject o, Activity a)
    {
        Constructor constr = getChallengeConstructor(classe);
        try {
            return ((Challenge) constr.newInstance(g, o, a));
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
        return null;
    }

}
